
/* Properties */


class Data {
   // Regular Property
   var fileName = "data.txt"

   // Lazy Properties
   lazy var biggy = [Int](count: 100, repeatedValue: 99)
}

var d = Data()
d.biggy
d


// lazy property is useful when:
//    1. Its value cannot be determined at the time of initialization
//    2. Calculating its value is computationally expensive or involves lengthy IO.

// Global variables are always lazy
// Local variables are always not lazy









// Computed Properties vs Stored Properties
class Rect {
   var width:Double = 1
   var height:Double = 1  // Stored Properties
   var area:Double {    // computed property must be var
      get {
         return width * height
      }
//      set {    // default parameter for setter is "newValue"
//         height = newValue / width
//      }
   }
}

class Rect2 {
   var width:Double = 1
   var height:Double = 1
   var area:Double {    // a computed property without setter is readonly
      get {
         return width * height
      }
   }
   var area2:Double {    // shorthand readonly computed property
      return width * height
   }
}









// Property observers
class Rect3 {
   var width:Double = 1
   var height:Double = 1 {
      willSet(newHeight) {  // called before value is set
         print("Height will be set to \(newHeight)")
      }
      didSet {    // called after value is set
         print("Height is changed from \(oldValue) to \(height)")
      }
   }
}

let r = Rect3()
r.height = 4

// Note: 1. lazy properties cannot have observers.
//       2. If a property already have setter/getter, it can't have observers.



// Type Properties (static properties)
struct Rect5 {
   var width:Double = 1
   var height:Double = 1
   static var color:String = "White"
}
Rect5.color
Rect5.color = "Black"


class Rect4 {
   var width:Double = 1
   var height:Double = 1
//   class var color:String = "White"  // not yet supported
}












// Enumeration
enum Direction {

   static let p1:String = ""
   //let p2:Int = 5   // enum can't have stored properties
   var p3:Bool {      // enum can only have computed properties
      return true
   }
   case North
   case South
   case East
   case West
}

Direction.p1
let x = Direction.North
x.p3






















