
/* Initializer Inheritance */

/*
Subclass must provide default values for properties it introduced:

Rule 1 - Designated Initializer
If your subclass doesn’t define any designated initializer, it automatically
inherits all of its superclass designated initializers.

Rule 2 - Convenience Initializer
If your subclass provides an implementation of all of its superclass designated
initializers — either by inheriting them as per rule 1, or by providing a custom
implementation as part of its definition — then it automatically inherits all of
the superclass convenience initializers.
*/

class Vehicle {
   var type:String
   init(type:String) {
      self.type = type
   }
   
   convenience init(x:Int) {
      self.init(type: "Bike")
   }
}

class Bike : Vehicle {
   var age:Int = 8  // all properties have to be initialized
   
   init() {
      super.init(type: "Bike")
   }
   
   override init(type:String) {
      super.init(type:type)
   }
}

let b1 = Bike(type: "Bike")  // designated initializers are inherited
let b2 = Bike(x: 9)          // Convenience initializers are also inherited

















//class Bike : Vehicle {
//   var owner = "Unknown"
//   
//   // Initializers are not inherited by default
//   init() {
//      super.init(type: "Bike")
//   }
////   override init(type:String) { // this will satisfy the second rule
////      super.init(type: type)
////   }
////   convenience override init(type:String) {  // this also satisfy the second rul
////      self.init()
////   }
//}
//
////let b = Bike(type: "Bike")  // Error: Vehicle's init(String) is not inherited
////let bk = Bike(x:9)  // Convenience init is inherited





class Vehicle3 {   // has an auto generated init()
}

class Bike3 : Vehicle3 {
   override init() {

   }
}





/* Required Initializer */
// Subclass is required to have implementation of the initializer, either by inheriting them, or by providing a custom implementation as part of its definition

class ParentClass {
   required init(name:String) {}
}

class ChildClass : ParentClass {
//   init(age:Int) {
//      super.init(name:"Bob")
//   }
//   
//   required init(name:String) {
//      super.init(name:name)
//   }
}

let ch = ChildClass(name: "Bob")



class GrandChildClass : ChildClass {
      init(age:Int) {
         super.init(name: "Bob")
      }

      required init(name: String) {   // This must be provided since GrandChildClass has its own initializer
          fatalError("init(name:) has not been implemented")
      }
}














/* Initilaization with Closure */
// property needs to be computed at the time of creation
// unlike computed properties, which are computed everytime they are accessed.
import Foundation
class Vehicle2 {
   
   var type = "Unknown"
   
   var createTime:String = {
      var d = NSDateFormatter()
      d.dateFormat = "HH-mm-ss"
      return d.stringFromDate(NSDate())
   }()  // without parentheses, you are assigning a closure to description
   
//   var type = "Unknown"
//   var discription:String = { return "My type is \(self.type)" }()  
                               // Can't access self here
}
let v = Vehicle2()  // time changes every time playground reruns















