
/* Automatic Reference Counter */


class Dog {
    var name = ""
    var dh:DogHouse?
    init() { print("Dog is created") }
    deinit { print("Dog is destroyed") }
}

class DogHouse {
    var d:Dog?
}

var dg:Dog?

func foo(){
    print("Start")
    
    var d:Dog? = Dog()
    dg = d
    
    d = nil // Dog is destroyed here
    
    print("End")
}


func goo() {
    let dog = Dog()
    let house = DogHouse()
    dog.dh = house
    house.d = dog  // Both dog and house will not be destroyed -- memory leak
}



// solution 1: use weak reference
//class DogHouse {
//    weak var d:Dog?
    // d is set to nil when the Dog is distroyed
    // weak reference is always optional
    // weak reference must be var
//}

// solution 2: use unowned reference
//class DogHouse {
 //   unowned var d:Dog
    // unowned reference is always referencing a value (not nil)
    // unowned reference is always defined as a non-optional type.
    // unowned reference can be var or constant
//}


// 1. If the property can be nil    --> use weak reference
// 2. If the property cannot be nil --> use unown reference







// Closure
class HTMLElement {
    let name:String
    let text:String?
    
    lazy var toString: ()->String = {
        if let text = self.text {
            return "<\(self.name)>\(text)</\(self.name)>"
        } else {
            return "<\(self.name)>"
        }
    }
    
    init(name:String, text:String? = nil) {
        self.name = name
        self.text = text
    }
}

//Note:
// 1. toString is a closure
// 2. toString is a lazy property --> OK to use self.name and self.text
// 3. Reference cycle between HTMLElement instance and the closure.

var e:HTMLElement? = HTMLElement(name: "p", text: "Hello")
e = nil  // neither e nor its closure is destroyed









// Solution: Capture List
class HTMLElement2 {
    let name:String
    let text:String?
    lazy var toString: ()->String = {
        [unowned self] in  //
        if let text = self.text {
            return "<\(self.name)>\(text)</\(self.name)>"
        } else {
            return "<\(self.name)>"
        }
    }
    
    lazy var asHTML2: (String)->String = {
        [unowned self](message:String)->String in  //
        // Closure body
        return ""
    }
    
    // Use weak reference if the reference can be nil
    
    init(name:String, text:String? = nil) {
        self.name = name
        self.text = text
    }
}






