
/* Generics */

func swapTwoInts(inout a: Int, inout b: Int) {
   let temporaryA = a
   a = b
   b = temporaryA
}

var x = 3; var y = 6
swapTwoInts(&x, b: &y)  // x == 6, y == 3



// Generic Function
func swapTwo<T>(inout a: T, inout b: T) {
   let temporaryA = a
   a = b
   b = temporaryA
}

swapTwo(&x, b: &y)  // still works
var s1 = "Good"; var s2 = "Bad"
swapTwo(&s1, b: &s2)  // s1 == "Bad";  s2 == "Good"


// Note: T is called "type parameter"







// Generic Types
struct Stack<T> {
    var items = [T]()
    
    mutating func push(item:T) {
        items.append(item)
    }
    
    mutating func pop() -> T {
        return items.removeLast()
    }
}

var s = Stack<Int>()
s.push(45)
s.pop()


// Exension of Generic Types
extension Stack {  // The original type parameter is used
    var topItem:T? {
        return items.last
    }
}








// Type Constraints

class Animal {}
protocol Walkable {}

func foo<T:Animal,U:Walkable>(t:T, u:U) {
   // T must be an Animal or subclass of Animal
   // U must conform to Walkable
}

class Wolf : Walkable {}

// OK
foo(Animal(), u: Wolf())















// Associated Type for Protocols

protocol Container {
    typealias ItemType
    mutating func append(item:ItemType)
}

struct IntStack : Container {
    var items = [Int]()
    
    // Conform to protocol
    //typealias ItemType = Int  // This can be inferred
    mutating func append(item: Int) {
        items.append(item)
    }
}


struct GenericStack<T> : Container {
    var items = [T]()
    
    // Conform to protocol
    //typealias ItemType = T  // This can be inferred
    mutating func append(item: T) {
        items.append(item)
    }
}






// We have encountered generics before

let a = [2, 3, 4]  // a is Array<Int>

let d = [1:"Monday", 2:"Tuesday"] // d is Dictionary<Int, String>
















