/* Closures */

// Closure   -- Unnamed Function

func lessThan(a:Int, b:Int) -> Bool {
   return a < b
}

let c1 = lessThan
c1(2,b: 3)


var c2 = {    //  A Closure
   (a:Int, b:Int)-> Bool in
   return a < b
}
c2(2,3)




// Benefits of Closure

var arr = [2,4,1,3,5]
arr.sortInPlace()
arr.sortInPlace(lessThan)


// 1.
arr.sortInPlace({(a:Int, b:Int)->Bool in    // Closure
   return a < b
})

// 2.
arr.sortInPlace({a, b in
   return a < b
})

// 3.
arr.sortInPlace({a, b in a < b })

// 4.
arr.sortInPlace({$0 < $1})

// 5.
// Trailing closure
arr.sortInPlace() {$0 < $1}


// Opertors are functions
arr.sortInPlace(>)




// “Closures are self-contained blocks of functionality that can be passed around and used in your code”

// Functions -- Named Closure


arr.map(){
   (number) -> String in
   return "number = \(number)"
}














// Another big advantage of closure: capturing values
var x = 5
arr.map(){
   (number) -> String in
   x++   // Captured variable can even be modified
   return "number = \(number * x)"
}
x




// Reference Catpure vs. Value Capture
var i = 25

//var c = { println(i) }     // Capture a reference to i
// Capture list
var c = {[i] in print(i)}  // Capture value of i


i = 35
c()   // prints out 25


// Capture list should appear as the first item in closure
let c3 = {[i](d:Int)->String in
   return "\(d+i)"
}












// More complicated example from book
func makeIncrementor(amount amount: Int) -> () -> Int {
   var runningTotal = 0
   func incrementor() -> Int {
      runningTotal += amount
      return runningTotal
   }
   return incrementor
}

let inc1 = makeIncrementor(amount: 10)
inc1()
inc1()
inc1()

let inc2 = makeIncrementor(amount:10)
inc2()


// OOP: Data contain functions
// Closure: Functions contain data


// inc1 and inc2 are constants, but runningTotal is a variable
// Function is a reference type





